# Generated by roxygen2: do not edit by hand

S3method("$",ts_tree)
S3method("[",ts_tree)
S3method("[[",ts_tree)
S3method("[[<-",ts_tree)
S3method("ts_tree_select<-",ts_tree)
S3method(as.character,ts_caller_arg)
S3method(as.character,ts_tree)
S3method(as.raw,ts_tree)
S3method(format,ts_parse_error)
S3method(format,ts_tree)
S3method(print,ts_parse_error)
S3method(print,ts_tree)
S3method(ts_tree_ast,default)
S3method(ts_tree_dom,default)
S3method(ts_tree_mark_selection1,ts_tree)
S3method(ts_tree_new,ts_language)
S3method(ts_tree_query,default)
S3method(ts_tree_select1,default)
S3method(ts_tree_select1,ts_tree.NULL)
S3method(ts_tree_select1,ts_tree.character)
S3method(ts_tree_select1,ts_tree.integer)
S3method(ts_tree_select1,ts_tree.logical)
S3method(ts_tree_select1,ts_tree.numeric)
S3method(ts_tree_select1,ts_tree.ts_tree_selector_default)
S3method(ts_tree_select1,ts_tree.ts_tree_selector_ids)
S3method(ts_tree_select1,ts_tree.ts_tree_selector_regex)
S3method(ts_tree_select1,ts_tree.ts_tree_selector_tsquery)
S3method(ts_tree_sexpr,default)
S3method(ts_tree_write,default)
export("ts_tree_select<-")
export(as_ts_caller_arg)
export(ts_caller_arg)
export(ts_caller_env)
export(ts_check_named_arg)
export(ts_cnd)
export(ts_collapse)
export(ts_list_parsers)
export(ts_parse_error_cnd)
export(ts_tree_ast)
export(ts_tree_delete)
export(ts_tree_deleted)
export(ts_tree_dom)
export(ts_tree_format)
export(ts_tree_insert)
export(ts_tree_mark_selection1)
export(ts_tree_new)
export(ts_tree_query)
export(ts_tree_select)
export(ts_tree_select1)
export(ts_tree_selected_nodes)
export(ts_tree_selection)
export(ts_tree_sexpr)
export(ts_tree_unserialize)
export(ts_tree_update)
export(ts_tree_write)
useDynLib(tsitter, .registration = TRUE, .fixes = "c_")
