## usethis namespace: start
#' @useDynLib tsitter, .registration = TRUE, .fixes = "c_"
## usethis namespace: end
NULL

## usethis namespace: start
## usethis namespace: end
NULL

#' @name about
#' @title About tsitter
#' @description
#' tsitter is a common interface to [tree-sitter](
#'  https://tree-sitter.github.io/tree-sitter/) parsers, implemented in
#' other R packages. It has a common API to
#'
#' - query,
#' - edit,
#' - format, and
#' - unserialize
#'
#' tree-sitter parse trees.
#'
#' @return
#' Not applicable.
#'
#' @examples
#' # See above please.
#'
#' @details
#'
#' ```{r, child = "tools/man/about.Rmd"}
#' ```
NULL
