#| title: 
# ---
This is the engine of \code{\link[tsitter:print.ts_tree]{print.ts_tree()}},
possibly useful to obtain a printed representation without doing the
actual printing.\if{html}{\out{
</p><p><!--comment-->
}}


If there are selected nodes in the tree, those will be highlighted
in the output. See \code{\link[tsitter:ts_tree_select]{ts_tree_select()}} to
select nodes in a tree.\if{html}{\out{
</p><p><!--comment-->
}}


\eval{tsitter:::doc_tabs("format_examples")}
