#| title: 
# ---
\subsection{The syntax tree and the DOM tree}{

See \code{\link[tsitter:ts_tree_ast]{ts_tree_ast()}} for the complete
tree-sitter syntax tree that includes all nodes, including syntax
elements like brackets and commas.

\eval{tsitter:::doc_tabs("ts_tree_ast_details_syntax_vs_dom")}
}
