#| title: 
# ---
Character string, connection, or \code{NULL}. The file or connection
to write to. By default it writes to the same file that was used in
\code{\link[tsitter:ts_tree_new]{ts_tree_new()}}, if \code{tree} was read from a
file.
