#| title: 
# ---
Character vector, the formatted annotated syntax tree, line by
line. It has class \link[cli:tree]{cli_tree}, from the cli package. It
may contain ANSI escape sequences for coloring and hyperlinks.
