#| title: 
# ---
The \code{[[} operator works similarly to the combination of
\code{\link[tsitter:ts_tree_select]{ts_tree_select()}} and
\code{\link[tsitter:ts_tree_unserialize]{ts_tree_unserialize()}}, but it
might be more readable.
