#| title: 
# ---
Calls \code{\link[tsitter:format.ts_tree]{format.ts_tree()}} to format the
ts_tree object, writes the formatted object to the standard output, and
returns the original object invisibly.

\eval{tsitter:::doc_tabs("print_examples")}
