#| title: 
# ---
Can be used to parse part(s) of the input. It must be a data frame with
integer columns \code{start_row}, \code{start_col}, \code{end_row}, \code{end_col},
\code{start_byte}, \code{end_byte}, in this order.
